use crate::ui::objects::app::AppObject;

use std::cell::RefCell;
use std::option::Option;

use adw::prelude::*;
use adw::subclass::prelude::*;
use glib::once_cell::sync::OnceCell;
use glib::subclass::InitializingObject;
use gtk::{gio, glib, CompositeTemplate, ListBox, TemplateChild};

#[derive(CompositeTemplate, Default)]
#[template(resource = "/dev/fyloz/example/window.ui")]
pub struct Window {
    #[template_child]
    pub apps_list: TemplateChild<ListBox>,
    pub apps: OnceCell<gio::ListStore>,
    pub current_app: RefCell<Option<AppObject>>,
}

#[glib::object_subclass]
impl ObjectSubclass for Window {
    const NAME: &'static str = "ConfigManagerWindow";
    type Type = super::Window;
    type ParentType = adw::ApplicationWindow;

    fn class_init(klass: &mut Self::Class) {
        klass.bind_template();
    }

    fn instance_init(obj: &InitializingObject<Self>) {
        obj.init_template();
    }
}

impl ObjectImpl for Window {
    fn constructed(&self) {
        self.parent_constructed();

        let obj = self.obj();
        obj.setup_apps();
        obj.restore_data();
    }
}

impl WidgetImpl for Window {}

impl WindowImpl for Window {}

impl ApplicationWindowImpl for Window {}

impl AdwApplicationWindowImpl for Window {}
