mod imp;

use crate::ui::objects::app::{AppData, AppObject};

use adw::prelude::*;
use adw::subclass::prelude::*;
use gtk::{gio, glib, ListBoxRow, Label, pango};
use glib::{clone, Object};

glib::wrapper! {
    pub struct Window(ObjectSubclass<imp::Window>)
        @extends adw::ApplicationWindow, gtk::Window, gtk::Widget,
        @implements gio::ActionGroup, gio::ActionMap, gtk::Accessible, gtk::Buildable,
                    gtk::ConstraintTarget, gtk::Native, gtk::Root, gtk::ShortcutManager;
}

impl Window {
    pub fn new(app: &adw::Application) -> Self {
        Object::builder().property("application", app).build()
    }

    fn current_app(&self) -> AppObject {
        self.imp()
            .current_app
            .borrow()
            .clone()
            .expect("`current_app` should be set in `set_current_app`.")
    }

    fn apps(&self) -> gio::ListStore {
        self.imp()
            .apps
            .get()
            .expect("`apps` should be set in `set_apps`.")
            .clone()
    }

    fn create_app_row(&self, app_object: &AppObject) -> ListBoxRow {
        let label = Label::builder()
            .ellipsize(pango::EllipsizeMode::End)
            .xalign(0.0)
            .build();

        app_object
            .bind_property("name", &label, "label")
            .sync_create()
            .build();

        ListBoxRow::builder().child(&label).build()
    }

    fn set_current_app(&self, app: AppObject) {
        self.imp().current_app.replace(Some(app));
    }

    fn select_collection_row(&self) {
        if let Some(index) = self.apps().find(&self.current_app()) {
            let row = self.imp().apps_list.row_at_index(index as i32);
            self.imp().apps_list.select_row(row.as_ref());
        }
    }

    fn setup_apps(&self) {
        let apps = gio::ListStore::new::<AppObject>();
        self.imp()
            .apps
            .set(apps.clone())
            .expect("Could not set apps");

        self.imp().apps_list.bind_model(
            Some(&apps),
            clone!(@weak self as window => @default-panic, move |obj| {
                let app_object = obj
                    .downcast_ref()
                    .expect("The object should be of type `AppObject`.");
                let row = window.create_app_row(app_object);
                row.upcast()
            }),
        )
    }

    fn restore_data(&self) {
        let data = vec!(
            AppData {
                name: "IntelliJ".to_string(),
                path: "/home/william/.config/jetbrains/intellij".to_string(),
            },
            AppData {
                name: "Spotify".to_string(),
                path: "/home/william/.config/spotify".to_string(),
            });

        let apps: Vec<AppObject> = data.iter()
            .map(AppObject::from_app_data)
            .collect();

        self.apps().extend_from_slice(&apps);

        if let Some(app) = apps.first() {
            self.set_current_app(app.clone());
        }
    }
}
